Cver.LogLik.HMSAR<-function(data, theta, raw.init.vec, reduc=FALSE){
  #Function to perform the Forwards algorithm for Hamilton's Markov Switching AR model
  #This function uses  C based code to compute the likelihood

#data=time series analysed
#theta=the parameter sample that we are computing the likelihood for
#raw.init.vec=probability initialisation vector for underlying MC
#reduc=is a reduction performed on the expanded SS
  
  #Orginal theta information
  X.TP.mat<-as.matrix(theta[[1]]) #Transition probability matrix, assumed to be first order
  no.X<-nrow(X.TP.mat) #Number of underlying states assumed in HMM
  X.SS<-colnames(X.TP.mat) #Colnames
  X.SS.mat<-matrix(as.numeric(unlist(strsplit(X.SS, ","))), nrow=no.X, byrow=T)
  
  mu.vec<-theta[[2]] #State dependent means
  prec.vec<-theta[[3]]; var.vec<-1/prec.vec; sd.vec<-sqrt(var.vec)
  #State dependent precision, variances, sd
  PAC.vec<-theta[[4]]; AR.vec<-PacfToAR(PAC.vec)
  #The PAC, converted to the respective AR coefficients which are much more useful
  #Ordering of AR parameters will be AR1, AR2, ... (most recent to past)
  
  #Now transform onto the necessary space e.g. m-th order MC with respect to AR dependency
  rval<-length(AR.vec) #Order of dependency induced by the AR, could of course be 0th order but the AR parameter will be 0
  
  if(reduc==FALSE){
#Reduction on SS is not performaned
#This section basically considers what are the possible moves between states and thus 
                   Xrtup.TP.mat<-tup.to.rtup(X.TP.mat, rval)
                   Xrtup.TP.vec<-as.vector(t(Xrtup.TP.mat)) #Transition prob matrix is vector form
                   rtuple.states<-colnames(Xrtup.TP.mat) #Corresponding states
                   no.rtuples<-length(rtuple.states)
                   rtuple.states.mat<-matrix(as.numeric(unlist((strsplit(rtuple.states, ",")))), nrow=no.rtuples, byrow=T)
                   
                   }#The transition matrix defined over this new order
  else{
#This does consider a reduction on the SS
#Reduction is achieved by not allowing frequent switches between states
reduc_info<-base.to.reduc.TP.mat(base.TP.mat=X.TP.mat, order.m=rval)
       Xrtup.TP.mat<-reduc_info[[1]] #r-tuple TP matrix defined over a reduced SS which doesn't allow for multipled quick switches within tuple time window
       Xrtup.TP.vec<-as.vector(t(Xrtup.TP.mat))
       rtuple.states<-reduc_info[[3]]#Corresponding states
       no.rtuples<-length(rtuple.states)
       rtuple.states.mat<-reduc_info[[2]]
       } 
  
   rtuple.init.vec<-rep(raw.init.vec/(no.rtuples/no.X), each=no.rtuples/no.X) #New initial vector defined over the r-tuples
  
  ext.mu.mat<-matrix(mu.vec[rtuple.states.mat], nrow=no.rtuples, ncol=rval, byrow=F)
  ext.mu.vec<-as.vector(t(ext.mu.mat)) #Corresponding mu's over extended statespace
  
  ext.sd.mat<-matrix(sd.vec[rtuple.states.mat], nrow=no.rtuples, ncol=rval, byrow=F)
  ext.sd.vec<-rowMeans(ext.sd.mat) #The corresponding effective sds over the extended statespace
  
  data.T<-length(data)#Length of log-lik
  
  dyn.load("HMSAR_LogLik_Cpart.so")
  LogLik<-.C("Cver_LogLik_HMSAR", as.double(data), as.integer(data.T), as.double(Xrtup.TP.vec), as.double(ext.mu.vec), as.double(ext.sd.vec), as.double(AR.vec), as.integer(rval), as.integer(no.rtuples), as.double(rtuple.init.vec), "li-lo"=as.double(0), double(data.T)) #Call the C function to do all the hardwork
  
  LogLik[["li-lo"]]
  #Log-lik as output
}


#######


