#Complete SMC samplers algorithm for Gaussian Markov Mixture data
	#General with respect to specifying any number of states we want
		
Gen_SMC_complete_GaussMM<-function(N=1000, B=100, temp.sched, data, prop.var.vec, corr.tp=-0.75, hyper.vec, X.init.vec=NULL, no.X=2){
#SMC samplers for a Gaussian Markov Mixture data sequence
	#C based version due to the likelihood calculation
    #N= number samples used to approximate the distributions in question
    #B = number of distributions to be approximated, i.e. number of iteration steps to move from prior to posterior
    #temp.sched = specified tempering schedule which controls how quickly we move between iterations /how dissimilar they are
        #Default is a simple linear schedule
#data=a vector of the time series in question
#prop.var.vec= the proposal variances vector for the mutation step (transition probability, means, precision)
    #hyper.vec= hyper parameters for transition probabilities, mean and precisions
    #arguments regarding the transform performed on the time series
   
if(is.null(X.init.vec)==TRUE){X.init.vec<-c(1, rep(0, no.X-1))} #Default initisation vector to use
	
#Initialisation, obtain a starting set of particles/samples, log lik and their weights
SMC_Init_info<-Gen_SMC_intialisation_GaussMM(N=N, data=data, hyper.vec=hyper.vec, init.vec=X.init.vec, no.X=no.X)
init.samples<-SMC_Init_info$I_Samples
init.loglik<-SMC_Init_info$I_Log; init.weights<-SMC_Init_info$I_Weights

init.samples.mat<-matrix(init.samples, nrow=N, ncol=1, byrow=F) #Matrix where each row represents a different sample
    #The idea is to store all samples in an array where the 3rd dimension represents the iterations step

#Storage
particle.storage.array<-array(init.samples.mat, dim=c(N, 1, B)) # In array form so that the 3rd dimensions represents the iteration
loglik.mat<-t(as.matrix(init.loglik)) #Each row of log lik and weights mat will represent different iterations, each column represents a different sample
nweights.mat<-t(as.matrix(init.weights))
resample.vec<-NULL #Vector containing the resampling iterations
NC.vec<-NULL #Vector containing the normalising constants that we use to approximate the 

#Mutation and Reweighting Reiteration step
for(iter in 2:B){
temperature<-temp.sched[iter] #The associated temperature
curr.part.mat<-particle.storage.array[,,iter-1] #The current particle set for this iteration
curr.loglik.vec<-loglik.mat[iter-1,] #The current log-lik for this iteration
curr.nweights.vec<-nweights.mat[iter-1,] #The current normalised weights for this iteration

#Resample?
ESS<-1/sum(curr.nweights.vec^2) #Effective sample size
if({ESS<N/2} & {iter !=B}){ #Is resampling necessary?
Resample_indices<-System_Resample(index.vec=1:N, w.vec=curr.nweights.vec) #Resampling indices particles
curr.part.mat<-curr.part.mat[Resample_indices] #Extract the appropriate "strong" particles
curr.loglik.vec<-curr.loglik.vec[Resample_indices] #The corresponding log-lik for the resampled sampled
curr.nweights.vec<-1/N #Equally weight the resampled particles
resample.vec<-c(resample.vec, iter) #Add the iteration time to the set of resample time points
}

inc.weights<-exp((temperature-temp.sched[iter-1])*curr.loglik.vec) #Incremental weights, based on the previous particle's log-likelihood
n.weights<-curr.nweights.vec*inc.weights #Unnormalised weights...
NC<-sum(n.weights); NC.vec<-c(NC.vec, NC) #Normalising constants and collate for approximating the normalising constant of the distributions in question
n.weights<-n.weights/NC #...now normalised weights
nweights.mat<-rbind(nweights.mat, n.weights) #Add to storage

#Actual mutation procedure
for(i in 1:N){
curr.part<-curr.part.mat[[i]]; curr.loglik<-curr.loglik.vec[i] #The current particle and its loglik in question
mutation_info<-Gen_SMC_mutation_GaussMM(curr.theta=curr.part, curr.li=curr.loglik, prop.var.vec=prop.var.vec, hyper.vec=hyper.vec, temp=temperature, data=data, init.vec=X.init.vec, iter=iter, corr.tp=corr.tp, no.X=no.X)
	#Perform a mutation on the current sample at this iteration
curr.part.mat[[i]]<-mutation_info[[1]] #Add new mutated sample to particle matrix
curr.loglik.vec[i]<-mutation_info[[2]] #Similarly for the log likelihood
}

#Adding everything storage (samples, log lik) and reweighting accordingly
particle.storage.array[,,iter]<-curr.part.mat
loglik.mat<-rbind(loglik.mat, curr.loglik.vec)

}

list(curr.part.mat, n.weights, curr.loglik.vec, NC.vec) 
#Output, a list of the final samples approximating the posterior 
#The associated weights of these samples
#The associated log-lik of these samples
#A vector of the normlising constants concerning the weights. Of which prod(NC.vec) provides the normalising constant for the parameter posterior.
}
