#SMC sampler for Gaussian Markov Mixture, general by specifying any number of states we want.

#The initialisation step which involves sampling from the prior.
Gen_SMC_intialisation_GaussMM<-function(N=1000, data, hyper.vec, init.vec=NULL, no.X=2){
#Performing the entire initialisation process of the SMC samplers
    #Prior sampling, likelihood calculation, weights

#N=number of samples used to approximated the prior
#data= the time series of interest
#hyper.vec= hyperparameter vector for TP, means and precisions
#init.vec= initisation vector for the underlying states
#no.X= number of underlyingstates we assume in our HMM

N.star<-N
init.log.lik.vec<-NULL; init.theta.list<-NULL
if(is.null(init.vec)==TRUE){init.vec<-rep(0, no.X); init.vec[1]<-1}
while(N.star!=0){ #This whole procedure ensures the initial samples we begin with have a defined likelihood, and that we obtain the number of samples required

Raw.particles<-Gen_SMC_prior_sampling_GaussMM(N=N.star, hyper.vec=hyper.vec, no.X=no.X)
	#Generate sample from the prior

#Compute the log lik of each sample
Raw.log.lik<-sapply(Raw.particles, FUN=Cver.LogLik.Gauss.MM, data=data, raw.init.vec=init.vec)  

bad.index<-which(is.nan(Raw.log.lik)==TRUE) #Which are bad samples with undefined log likelihood
if(length(bad.index)!=0){ #The case when bad samples exist

#Add good samples to appropriate lists
init.theta.list<-c(init.theta.list, Raw.particles[-bad.index])
init.log.lik.vec<-c(init.log.lik.vec, Raw.log.lik[-bad.index])
N.good<-N.star-length(bad.index) #How many good samples are there?
N.star<-N.star-N.good #New number of samples that we need to obtain for on next iteration
}
else{ init.theta.list<-c(init.theta.list, Raw.particles) #The case when there are no bad samples at all
init.log.lik.vec<-c(init.log.lik.vec, Raw.log.lik)
N.good<-N.star-length(bad.index) #How many good samples are there?
N.star<-N.star-N.good #New number of samples that we need to obtain in next loop
}}
list("I_Samples"=init.theta.list, "I_Log Lik"=init.log.lik.vec, "I_Weights"=rep(1, N)/N)
#Output, a list of samples from the prior, their associated log likelihood and their weights (equally weighted since we are able to sample them directly from the prior)
}



#SMC_prior_sampling
Gen_SMC_prior_sampling_GaussMM<-function(N=1000, hyper.vec, no.X=2){
#Prior sampling coomponent of the SMC samplers for a Gaussian Markov Micture
#General as we can specify however many 

#N= no of samples used to approximate distributions
#hyper.vec =hyper parameter vector for model parameters (TP, means, precisions)
#no.X= number of underlying states we assume in our 


hyper.info<-hyper.vec.sep(hyper.vec, no.X=no.X)
tp.hyper<-hyper.info[[1]] #Hyperparameter for transition probabilities, strength in a Dirichlet
mu.hyper<-hyper.info[[2]] #Hyperparameters for emission mean,  mean and variance for Gaussian prior
prec.hyper<-hyper.info[[3]] #Hyperparameters for the emission precisions, shape and scale respectively                                                

#Transition Probability sampling
tp.hyper.mat<-matrix(1,nrow=no.X, ncol=no.X) #Create a vector containing the corresponding hyper parameters
diag(tp.hyper.mat)<-tp.hyper #Replace with hyperparameter strength specified

tp.init.array<-array(1, dim=c(no.X, N, no.X)) #An array to store the intialisation tranisition probability matrix information
	#Dim 1= , Dim 2= sample, Dim 2= TP vectors sampled according to that Dirichlet prior
for(row in 1:no.X){
tp.init.array[,,row]<-dirchlet.sample(shape.vec=tp.hyper.mat[row,] , no.X=no.X, no.samp=N) #Sample some row transition probabilities with respect to corresponding Dirichlet prior
}

#Mean sampling
mu.init.mat<-matrix(rnorm(N*no.X, mean=mu.hyper[1], sd=sqrt(mu.hyper[2])), nrow=no.X, ncol=N)
	#Matrix containing sampled mu's from prior. Each column denotes a different sample, each row is the associated mean for that particular state
#May want to edit so that we can have different hyperparameters for the mean corresponding to each state
	
#Precision sampling
lambda.init.mat<-matrix(rgamma(N*no.X, shape=prec.hyper[1], scale=prec.hyper[2]), nrow=no.X, ncol=N)
	#Matrix containing sampled lambda's from prior. Each column denotes a different sample, each row is the associated precision for that particular state.
#Of course, we could also have different Gamma priors for each state

X.SS<-1:no.X
theta.list<-vector("list", N)
for(part in 1:N){ #Constructing the particles
tp.mat<-matrix(tp.init.array[,part,], nrow=no.X, ncol=no.X, byrow=T, dimnames=list(X.SS, X.SS)) #Create a tranistion matrix what we've sampled so far

theta<-list("TP matrix"=tp.mat, "Means"=mu.init.mat[,part], "Precisions"=lambda.init.mat[,part]) #Form the appropriate theta by extracting the corresponding means and thetas as well
theta.list[[part]]<-theta
}
theta.list #Output= a list of samples from the prior distribution
}

########

###
hyper.vec.sep<-function(hyper.vec, no.X=2){
#Function to separate the hyper parameters for transition probability vectors, means and precisions

tp.hyper<-hyper.vec[1]
mu.hyper<-hyper.vec[2:3]
prec.hyper<-hyper.vec[4:5]

list("TP"=tp.hyper, "MU"=mu.hyper, "PREC"=prec.hyper)
}
