#Mutation algorithm SMC Gauss MM for general number of states


Gen_SMC_mutation_GaussMM<-function(curr.theta, curr.li, prop.var.vec, hyper.vec, temp, data, init.vec, iter, corr.tp, no.X=2){
#Function to perform the mutation of a particle given the current particle and its associated likelihood
    #For general number of states in the HMM

#curr.theta=the current sample to be mutated.
#curr.li=the likelihood of the current sample
#prop.var.vec=the proposal variances used as part of the mutation
#hyper.vec=the hyperparameters used
#temp=the current temperature
#data=the time series analysed
#init.vec=the initialisation vector of the underlying states
#iter=the current iteration we are on
#corr.tp=the correlation induced in the transition probability mutation (redundant here)
#no.X= the number of underlying states assumed in the HMM
       
tp.prop.var<-prop.var.vec[1] #Proposal variance for TP
mu.prop.var<-prop.var.vec[2] #Proposal variance for mean
prec.prop.var<-prop.var.vec[3] #Proposal variance for precision

#Hyper parameters
hyper.info<-hyper.vec.sep(hyper.vec, no.X=no.X)
tp.hyper<-hyper.info[[1]]
	tp.hyper.mat<-matrix(1, nrow=no.X, ncol=no.X); diag(tp.hyper.mat)<-tp.hyper #Hyperparameters for TP
mu.hyper.vec<-hyper.info[[2]] #Hyperparameters for emission mean
lambda.hyper.vec<-hyper.info[[3]] #Hyperparmeters for emission precision

#Transition Probability 
curr.tp.mat<-curr.theta[[1]]
prop.tp.mat<-TPmat.proposal.logit(curr.TPmat=curr.tp.mat, prop.var=tp.prop.var)
#A proposed transition probability matrix by mutating the existing sample
#Proposals based on RWMH on logit scale

#Setting up the particle so that we accept and reject each row of TP matrix one at a time
upd.theta<-curr.theta #Most recent sample value, with mutations having being performed along the way
upd.li<-curr.li #The associated likelihood

for(tp.row.index in 1:no.X){ #Test each TP row one by one
temp.prop.tp.mat<-upd.theta[[1]] #Extract the most recent version of the Transition Probability Vector
temp.prop.tp.mat[tp.row.index,]<-prop.tp.mat[tp.row.index,] #Add the suggested proposal TP row vector 

acc_rej.info<-Gen_SMC_TP.AccRej_GaussMM(curr.theta=upd.theta, curr.li=upd.li, prop.tp.mat=temp.prop.tp.mat, row=tp.row.index, phi=temp, data=data, init.vec=init.vec, tp.hyper.prior=tp.hyper.mat[tp.row.index,])
	#Performance acceptance or rejection on this proposed sample
upd.theta<-acc_rej.info[[1]] #The mutated sample
upd.li<-acc_rej.info[[2]]# The assoiated likelihood
}

#Each mean, proposal at the same time (independently), accept and reject separately
curr.mean.vec<-curr.theta[[2]] #Extract the current mean value
prop.mean.vec<-curr.mean.vec + rnorm(no.X, mean=0, sd=sqrt(mu.prop.var)/(iter-1))  #Proposals for the emission means, via RWMH

for(mean.index in 1:no.X){ #Testing each mean separately
inter.curr.mean.vec<-upd.theta[[2]] #Current emission mean vector
inter.prop.mean.vec<-inter.curr.mean.vec
    inter.prop.mean.vec[mean.index]<-prop.mean.vec[mean.index] #Add the proposal mean in the corresponding place
    
acc_rej.info<-Gen_SMC_mean.AccRej_GaussMM(curr.theta=upd.theta, curr.li=upd.li, prop.mean.vec=inter.prop.mean.vec, phi=temp, data=data, init.vec=init.vec, mean.prior.hyper=mu.hyper.vec) #Acceptance/rejection of the proposed mean vector

upd.theta<-acc_rej.info[[1]] #Mutated sample
upd.li<-acc_rej.info[[2]] #Associated likelihood
}

#Each precision, proposal at the same time (independently), accept and reject separately
curr.prec.vec<-curr.theta[[3]]
prop.prec.vec<-exp(log(curr.prec.vec)  +rnorm(no.X, mean=0, sd= sqrt(prec.prop.var/(iter-1)))) #Proposals for precision vector via RWMH on log-normal scale

for(prec.index in 1:no.X){
inter.curr.prec.vec<-upd.theta[[3]] #Intermediary current theta, this takes into account the most recent values
inter.prop.prec.vec<-inter.curr.prec.vec
    inter.prop.prec.vec[prec.index]<-prop.prec.vec[prec.index] #Add the proposal precision in the corresponding place
    
acc_rej.info<-Gen_SMC_prec.AccRej_GaussMM(curr.theta=upd.theta, curr.li=upd.li, prop.prec.vec=inter.prop.prec.vec, phi=temp, data=data, init.vec=init.vec, prec.prior.hyper=lambda.hyper.vec)
   #Perform the acceptance rejection procedure for the proposed sample
upd.theta<-acc_rej.info[[1]] #The mutated sample
upd.li<-acc_rej.info[[2]] #The associated likelihood
}

list("Mutated sample"=upd.theta, "Mutated log-lik"= upd.li)
#Output, the mutated sample and it's associated likelihood
}


######
Gen_SMC_TP.AccRej_GaussMM<-function(curr.theta, curr.li, prop.tp.mat, row, phi, data, init.vec, tp.hyper.prior){
#Acceptance rejection procedure with respect to the joint TP proposals

#curr.theta=the current sample value
#curr.li = the associated likelihood
#prop.tp.mat= the proposed transition probability matrix
#row=the row of the TP matrix we are considering
#phi =current temperature we are at
#data=the time series we are considering
#init.vec=the initialisation vector for the underlying states
#tp.hyper.prior=the hyperparameter vector for the TP vector

#Construct the proposal theta
prop.theta<-curr.theta
prop.theta[[1]]<-prop.tp.mat #Create the proposal sample with the propsal TP matrix in place

curr.tp.mat<-curr.theta[[1]] #Current value of the TP matrix

#TP vectors that we are testing on
curr.tp.vec<-curr.tp.mat[row,] #Current TP row
prop.tp.vec<-prop.tp.mat[row,] #Proposal TP row

#Prior component
prop.prior<-dirchlet.log.density(xvec=prop.tp.vec, shape.vec=tp.hyper.prior) #Prior density of the proposal TP matrix
if(is.nan(prop.prior)==TRUE) {list(curr.theta, curr.li)}
else{
curr.prior<-dirchlet.log.density(xvec=curr.tp.vec, shape.vec=tp.hyper.prior) #Prior density of the current TP matrix
prior.comp<-prop.prior-curr.prior #Prior component of acceptance ratio

#Likelihood component
prop.li<-Cver.LogLik.Gauss.MM(data=data, theta=prop.theta, raw.init.vec=init.vec) #The log-lik under the proposal theta
if(is.nan(prop.li)==TRUE) {list(curr.theta, curr.li)}
else{
loglik.comp<-phi*(prop.li-curr.li) #Likelihood component of the acceptance ratio

#Additional Jacobian component due to proposal on a transformation
prop.jacob<--sum(log(prop.tp.vec)) #Moving TO the proposals FROM current
curr.jacob<--sum(log(curr.tp.vec)) #Moving TO the current FROM proposals
jacob.comp<-curr.jacob-prop.jacob

alpha<-min(1, prior.comp+loglik.comp+jacob.comp) #Acceptance probability, working on a log scale
uno<-log(runif(1))
if(uno<alpha){list(prop.theta, prop.li)} #Accept proposal
else{list(curr.theta, curr.li)} #Reject proposal and take through the existing particle
}}
}

#########
Gen_SMC_mean.AccRej_GaussMM<-function(curr.theta, curr.li, prop.mean.vec, phi, data, init.vec, mean.prior.hyper){
#Acceptance rejection procedure with respect to the means
#phi =current temperature we are at

#curr.theta=the current sample value
#curr.li = the associated likelihood
#prop.mean.vec= the proposed mean vector
#row=the row of the TP matrix we are considering
#phi =current temperature we are at
#data=the time series we are considering
#init.vec=the initialisation vector for the underlying states
#mean.prior.hyper=the hyperparameter vector for the mean vector

#Construct the proposal theta
prop.theta<-curr.theta
prop.theta[[2]]<-prop.mean.vec #Create the proposal theta based on the proposal mean

curr.mean.vec<-curr.theta[[2]] #Exract the current mean vector

#Prior component
prop.prior<-sum(dnorm(prop.mean.vec, mean=mean.prior.hyper[1], sd=sqrt(mean.prior.hyper[2]), log=T))#Prior density for the proposal mean vector
if(is.nan(prop.prior)==TRUE) {list(curr.theta, curr.li)}
else{
curr.prior<-sum(dnorm(curr.mean.vec, mean=mean.prior.hyper[1], sd=sqrt(mean.prior.hyper[2]), log=T))#Prior density for the current mean vector
prior.comp<-prop.prior-curr.prior #Prior component for the acceptance ratio

#Likelihood component
prop.li<-Cver.LogLik.Gauss.MM(data=data, theta=prop.theta, raw.init.vec=init.vec) #Likelihood under the proposal sample
if(is.nan(prop.li)==TRUE) {list(curr.theta, curr.li)} 
else{
loglik.comp<-phi*(prop.li-curr.li) #Likelihood component for the acceptance ratio

#No additional change of variable component since we worked on the usual scale and no transformation took place

alpha<-min(1, prior.comp+loglik.comp) #Acceptance probability, working on a log scale
uno<-log(runif(1))
if(uno<alpha){list(prop.theta, prop.li)} #Accept proposal
else{list(curr.theta, curr.li)} #Reject proposal and take through the existing particle
}}
}

############
Gen_SMC_prec.AccRej_GaussMM<-function(curr.theta, curr.li, prop.prec.vec, phi, data, init.vec, prec.prior.hyper){
#Acceptance rejection procedure with respect to the precisions

#curr.theta=the current sample value
#curr.li = the associated likelihood
#prop.mean.vec= the proposed mean vector
#row=the row of the TP matrix we are considering
#phi =current temperature we are at
#data=the time series we are considering
#init.vec=the initialisation vector for the underlying states
#prec.prior.hyper=the hyperparameter vector for the precision vector


#Construct the proposal theta
prop.theta<-curr.theta
prop.theta[[3]]<-prop.prec.vec #Proposal sample with proposal precision vector in place

curr.prec.vec<-curr.theta[[3]]

#Prior component
prop.prior<-sum(dgamma(prop.prec.vec, shape=prec.prior.hyper[1], scale=prec.prior.hyper[2], log=T)) #Prior density under the proposal precision
if(is.nan(prop.prior)==TRUE) {list(curr.theta, curr.li)}
else{
curr.prior<-sum(dgamma(curr.prec.vec, shape=prec.prior.hyper[1], scale=prec.prior.hyper[2], log=T)) #Preior density under the current precision
prior.comp<-prop.prior-curr.prior #Prior component for acceptance ratio

#Likelihood component
prop.li<-Cver.LogLik.Gauss.MM(data=data, theta=prop.theta, raw.init.vec=init.vec) #Likelihood under the proposed sample
if(is.nan(prop.li)==TRUE) {list(curr.theta, curr.li)}
else{
loglik.comp<-phi*(prop.li-curr.li) #Likleihood component of acceptance ratio

#Additional Jacobian component due to proposal on a transformation
prop.jacob<-sum(log(1/prop.prec.vec)) #At current, jumping to proposal
curr.jacob<-sum(log(1/curr.prec.vec)) #At proposal, jumping to proposal
jacob.comp<-curr.jacob-prop.jacob #Change of variables component of acceptance ratio

alpha<-min(1, prior.comp+loglik.comp+jacob.comp) #Acceptance probability, working on a log scale
uno<-log(runif(1))
if(uno<alpha){list(prop.theta, prop.li)} #Accept proposal
else{list(curr.theta, curr.li)} #Reject proposal and take through the existing particle
}}
}
