#include <R.h>  //Used for some debuggying via R
#include <Rdefines.h> // CN addition, from R-C inteface document,
#include <Rmath.h>
#include <Rinternals.h> // RK addition
#include <R_ext/RS.h>	// RK addition
#include <R_ext/Lapack.h> // RK addition
#include <R_ext/BLAS.h> // RK addition
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include <string.h>

// C script to perform the log-likelihood calculation for a HMS-AR model

void Cver_LogLik_HMSAR(double *data, int *ndata, double *TPvec, double *Meanvec, double *SDvec, double *ARvec, int *odep, int *Xsize, double *init_vec, double *lilo, double *scaling){
	// data = the data process we are analysing
	// ndata = length of the data process
	// TPvec = vector containing all transiton probabilities for TP matrix
	// Meanvec = vector containing all the state dependent means
	// SDvec =  vector  containing the state dependent SDs
	// ARvec = the predetermined AR parameters on previous observations
	// odep = order of dependency induced by AR 
	// Xsize = size of the underlying chains SS
	// init_vec = initialisation probability vector
	// lilo= log-likelihood that we want returned
	// scaling = scaling vector from which likelihood are computed from 

	//Note that the we'll probably be considering a m-th order MC as a 1st order MC here which probably confuses matters a bit more than usual.

int nr=*Xsize; //Size of the underlying statespace, the same as Xsize. Just so I don't get confused...

//TP mat construction
double TPmat[nr][nr]; 	
for(int i=0; i<nr; i++){ //To create a matrix given a vector of the elements
for(int j=0; j<nr; j++){
	TPmat[i][j] = TPvec[(nr *i)+j];
}
}	

//printf("TP 1: %f %f \n", TPmat[0][0], TPmat[0][1]);
//printf("TP 2: %f %f \n", TPmat[1][0], TPmat[1][1]);

//Mean mat construction
double Meanmat[nr][*odep];
int entry=0; //To go through each Mean coefficient
for(int i=0; i<nr; i++){ //To create a matrix given a vector of the elements
for(int j=0; j<*odep; j++){
	Meanmat[i][j] = *(Meanvec+entry);
	entry++;
}
}	

//printf("Mean Row1: %f, %f, %f, %f \n"  ,Meanmat[0][0], Meanmat[0][1], Meanmat[0][2], Meanmat[0][3]);
//printf("Mean Row2: %f, %f, %f, %f \n"  ,Meanmat[1][0], Meanmat[1][1], Meanmat[1][2], Meanmat[1][3]);
//printf("Mean Row3: %f, %f, %f, %f \n"  ,Meanmat[2][0], Meanmat[2][1], Meanmat[2][2], Meanmat[2][3]);

double emiss_den ;// Variable to store the emissision density

double prevobs[*odep] ;//Previous wavelet coefficient vector, we'll populate this in the for loop for each iteration
double interAR_vec[*odep];//Vector to store intermediary AR coefficients
double old_init_vec[nr];
double temp_init_vec[nr];

//printf("ARvev: %f, %f, %f, %f \n"  ,ARvec[0], ARvec[1], ARvec[2], ARvec[3]);
//printf("Initvec: %f, %f \n"  ,init_vec[0], init_vec[1]);

for(int iter=0; iter<*ndata; iter++){
//printf("Yello %f \n", iter);
for(int state=0; state<nr; state++){
//printf("State: %f \n"  ,state);
	old_init_vec[state] = init_vec[state]; //Store the old forward probabilities
//printf("Old_init: %f \n"  ,old_init_vec[state]);
}

for(int order=0; order<*odep; order++){
//printf("Order: %f \n"  ,order);
	interAR_vec[order] = ARvec[order]; //Store the old forward probabilities
//printf("stuff: %f \n"  ,interAR_vec[order]);
}


//Fill in the previous observations vector for this particular iteration
for(int ent=0; ent<*odep; ent++){
//printf("Hello %f \n", ent);
	if(iter-ent<0){
		prevobs[*odep-ent-1] =0; //For case when it the previous obs aren't technically available
			//RHS = most recent observations
		interAR_vec[ent] = 0; //Where no observation is present at that lag, put a 0 in the AR vector as well
	 } 
	else{
	prevobs[*odep-ent-1] =  *(data+iter-ent-1) ;//When the information is available
	}	//Minus 1s present due to C index, most recent obs is on the RHS
	//RHS = most recent observations
}

for(int j=0; j<nr; j++){ // j is the destination state
//Emission Denisty calculation

double update=0; //the previous forward and tp matrix calculation. Needs to be reset to 0 at each iteration

for(int i=0; i <nr; i++){

double mean_comp=0;

//ARvec= phi1, phi2, \ldots phir
for(int ent=0; ent<*odep; ent++){
mean_comp += interAR_vec[ent] * (prevobs[*odep-ent-1]-Meanmat[i][*odep-ent-1]) ;//The mean component of the emission density, based on our current state i
}
//printf("Mean Component: %f \n", (float) mean_comp);
emiss_den = dnorm(*(data+iter)-Meanmat[j][*odep-1], mean_comp, *(SDvec+j), 0) ;//Emission density under thus particular state, taking into account the previous observations

//printf("Emission: %f \n",emiss_den);
//printf("Inter: %f \n", *(old_init_vec+i) * TPmat[i][j]*emiss_den);

update += (*(old_init_vec+i) * TPmat[i][j]*emiss_den); //The new unnormalised forward probability for state j
	//\sum \alpha_i * p_{i,j} 
}

*(temp_init_vec+j) =  update ; //The unnormalised version of the forward probabilities, for this particular state j 
}

for(int i=0; i<nr; i++){
scaling[iter] += *(temp_init_vec+i); // Normalising constant
}

//printf("UnNorm forward: %f, %f, \n ", *(temp_init_vec+0), *(temp_init_vec+1));

//printf("Scaling: %f \n", scaling[iter]) ;
for(int i =0; i<nr; i++){
//printf("Prob: %f \n", *(temp_init_vec+i));
//printf("Scaling: %f \n", scaling[iter]) ;
*(init_vec+i) = *(temp_init_vec+i)/scaling[iter]; //Normalised version of the forward probabilities 
//printf("Prob: %f \n", *(init_vec+i));
}

//printf("Norm forward: %f, %f, \n ", *(init_vec+0), *(init_vec+1));

*lilo += log(scaling[iter]); //To compute the log-likelihood
}
}

//////////

