#Testing Script for HMSAR

setwd("/storage/striaj/AISM_paper_code/Functions_Data_Folder/") #Change working directy as necessary

source("Cver_HMSAR_LogLIk.r") #Files regarding log-lik computation of HMSAR
source("GenHMSAR_SMC_complete.r") #The main SMC sampler function
source("GenHMSAR_SMC_init.r") #Functions regarding the initialisation of the SMC sampler
source("GenHMSAR_SMC_mutation.r")#Functions regarding the mutation of the sampler

#HMSAR_LogLik_Cpart.c also needs to be compiled by the user

source("SMC_auxiliary.r") #Some general SMC supplementary functions e.g. Systematic resampling, TP proposals

source("Supplementary_Functions.r") #Some data generating functions and manipulation of TP matrices

#Packages
library(mvtnorm) #For multivariate normal
library(FitAR) #For PAC to AR conversion
library(compiler); enableJIT(3) #To speed up a few things
library(Matrix)

#Some simulated data to play around
Sim.HMSAR.mean.vec<-c(0,5) #State dependent means
Sim.HMSAR.SD.vec<-c(1, 1); Sim.HMSAR.prec.vec<-c(1, 1/Sim.HMSAR.SD.vec^2) #State dependent variances
Sim.HMSAR.AR.vec<-c(0.5);Sim.HMSAR.PAC.vec<-ARToPacf(Sim.HMSAR.AR.vec)
	#AR coefficients and PAC equivalents

Sim.UndMC<-c(rep(1,100), rep(2, 200)) #The underlying statespace we will simulate according to
Gen.data<-Sim.Data.Hamil.MSAR(X.UndMC=Sim.UndMC, mu.vec=Sim.HMSAR.mean.vec, sd.vec=Sim.HMSAR.SD.vec, AR.vec=Sim.HMSAR.AR.vec) #Simulated data according to HMSAR

plot(Gen.data, type="l")

#SMC samplers parameters
Gen.hyper.vec<-c(10, 0, 100, 1,1) #Hyperparameters for prior sampling
	#1st=TP Dirichlet Strength
	#2nd, 3rd = Emission Mean, mean and variance
	#4th, 5th = Emission Precision, shape and scale
Gen.prop.var<-c(10,10,10, 10)#Proposal variances for mutation step
	#1st=TP proposal variance
	#2nd=Emission mean variance
	#3rd=Emission precision variance
	#4th=AR/PAC proposal variance
	
Gen.N<-500; Gen.B<-100 #Number of samples and the distributions considered
Gen.ARval<-1 #AR order assumed in analysis

max.no.states<-3 #Max number of underlying states considered

NC.no.states<-NULL
for(no.states in 1:max.no.states){ #Not that this isn't the most efficient way of performing this. Consider parallel computing
no.states
Gen.SMC_info<-SMC_complete_HMSAR_GenStates(N=Gen.N, B=Gen.B, temp.sched=seq(0,1, len=Gen.B), data=Gen.data, prop.var.vec=Gen.prop.var, hyper.vec=Gen.hyper.vec, X.init.vec=c(1, rep(0,no.states-1)), rval=Gen.ARval, no.X=no.states, OriginalModel=TRUE)

NC.no.states<-c(NC.no.states, sum(log(Gen.SMC_info[[4]]))) #Normalising constants to be used in approximating model posterior. Individual elements are approximates of log of the marginal likelihood
}

NC.unlog<-exp(NC.no.states)
post.no.states<-NC.unlog/sum(NC.unlog)#Approximation of the model posterior

barplot(post.no.states, names.arg=1:max.no.states, ylab="Posterior", xlab="No. States") #Barplot of the posterior

##########################
#GNP data analysis
#GNP data
GNP.data<-scan("GNP82.DAT") #Scan in the raw data
GNP.data.trans<-diff(log(GNP.data))*100 #Perform the necessary transformations
GNP.data.trans<-ts(GNP.data.trans, start=1951.25, end=1984.75, freq=4) #In the time series format

GNP.rval<-4
GNP.max.states<-3

NC.no.states<-NULL
for(no.states in 1:GNP.max.states){ #Not that this isn't the most efficient way of performing this. Consider parallel computing
Gen.SMC_info<-SMC_complete_HMSAR_GenStates(N=Gen.N, B=Gen.B, temp.sched=seq(0,1, len=Gen.B), data=GNP.data.trans, prop.var.vec=Gen.prop.var, hyper.vec=Gen.hyper.vec, X.init.vec=rep(1, no.states)/no.states, rval=GNP.rval, no.X=no.states, OriginalModel=TRUE, reduc=TRUE)

NC.no.states<-c(NC.no.states, sum(log(Gen.SMC_info[[4]]))) #Normalising constants to be used in approximating model posterior. Individual elements are approximates of log of the marginal likelihood
}


NC.unlog<-exp(NC.no.states)
post.no.states<-NC.unlog/sum(NC.unlog)#Approximation of the model posterior

barplot(post.no.states, names.arg=1:GNP.max.states, ylab="Posterior", xlab="No. States") #Barplot of the posterior

