#Testing Script for HMSAR

setwd("/storage/striaj/AISM_paper_code/Functions_Data_Folder/") #Change working directy as necessary

source("Cver_GaussMM_LogLik.R") #Files regarding log-lik computation of GMM
source("Gen_GMM_SMC_complete.R") #The main SMC sampler function assuming a GMM
source("Gen_GMM_SMC_init_func.R") #Functions regarding the initialisation of the SMC sampler
source("Gen_GMM_SMC_mutation.R")#Functions regarding the mutation of the sampler

#C file GaussMM.c also needs to be compiled 

source("SMC_auxiliary.r") #Some general SMC supplementary functions e.g. Systematic resampling, TP proposals

source("Supplementary_Functions.r") #Some data generating functions and manipulation of TP matrices

#Some data to play around with
Gen.XUnd.MC<-c(rep(1, 100), rep(2, 150))
Gen.mean.vec<-c(0,0)
Gen.sd.vec<-c(1,2)
Gen.GMM.data<-Sim.Data.GaussMM.Orderm(X.UndMC=Gen.XUnd.MC, order.m=1, mu.vec=Gen.mean.vec,sd.vec=Gen.sd.vec)
plot(Gen.GMM.data, type="l")


#SMC samplers 
Gen.hyper.vec<-c(10, 0, 100, 1,1) #Hyperparameter vector
	#1st=TP Dirichlet prior strength
	#2nd, 3rd= Emission mean, mean and variance
	#4th, 5th= Emission precision, shape and scale

Gen.prop.var<-c(10,10,10)#Proposal variance vector
	#1st=TP
	#2nd=Mean
	#3rd=Precision

Gen.N<-500; Gen.B<-100 #Number of samples and distributions in the sequence
Gen.max.states<-3 #Max number of states considered in HMM

NC.no.states<-NULL
for(no.states in 1:Gen.max.states){ #You would do this via parallel computing in reality
Gen.SMC_info<-Gen_SMC_complete_GaussMM(N=Gen.N, B=Gen.B, temp.sched=seq(0, 1, length=Gen.B), data=Gen.GMM.data, prop.var.vec=Gen.prop.var, corr.tp=-0.75, hyper.vec=Gen.hyper.vec, X.init.vec=NULL, no.X=no.states)
NC.no.states<-c(NC.no.states, sum(log(Gen.SMC_info[[4]]))) #An vector containg an approximation of the marginal likelihood
}

NC.unlog<-exp(NC.no.states)
post.no.states<-NC.unlog/sum(NC.unlog) #Approximation of the model posterior

barplot(post.no.states, names.arg=1:Gen.max.states, ylab="Posterior", xlab="No. States")
